/*
 * UNC0xx GUI-based software. (c) Unicontrollers, Ltd., 2012
 *
 * Please see the files README, COPYING, LICENSE and WARRANTY
 * for some useful details on the legal status of this code.
 * Visit the site http://www.unicontrollers.com for more info.
 *
 * file demo_dev.cpp
 * Communication with the device for GUI demo program
 *
 * Written by Andrey V. Stolyarov
 *
 */

#include <stdio.h>

#include <uncusb.h>
#include <unc_data.h>

#include "demo_dev.hpp"


UNCDevice::~UNCDevice()
{
    if(dev_hdl)
        unc001_close(dev_hdl);
}

bool UNCDevice::OpenDevice()
{
    if(dev_hdl) {
        unc001_close(dev_hdl);
        dev_hdl = 0;
    }
    struct unc001_item *list;
    unc001_usb_init();
    list = unc001_scan(0);
    if(!list)
        return false;
    dev_hdl = list->dev_hdl;
    list->dev_hdl = 0;
    unc001_destroy_list(list);
    return true;
}

bool UNCDevice::Poll(int *state, char temp[32], char tabl[32])
{
    if(!dev_hdl) return false;

    struct unc001_global_state_str devstat;
    struct unc001_extra_state_str extra;
    int r;

    r = unc001_get_fullstate(dev_hdl, &devstat);
    if(r == -1) {
        return false;
    }
    *state = devstat.state_bits;

    temp[0] = 0;
    tabl[0] = 0;
    r = unc001_get_extrastate(dev_hdl, &extra);
    if(r != -1) {
        for(int i=0; i<4; i++) {
            if(extra.ow_ids[i][0]) {
                int j, itemp, grad, migrad;
                char *p;
                char tsign = '+';
                itemp = extra.ow_temperature[i];
                switch(itemp) {
                case (short)0x8000: /* no temperature here => tablet? */
                    if(extra.ow_ids[i][0] != 1) {
                        // not a tablet
                        break;
                    }
                    p = tabl;
                    sprintf(p, "%02X:", extra.ow_ids[i][7]);
                    p+=3;
                    for(j=6; j>=1; j--) {
                        sprintf(p, "%02X", extra.ow_ids[i][j]);
                        p += 2;
                    }
                    sprintf(p, ":%02X", extra.ow_ids[i][0]);
                    p+=3;
                    *p = 0;
                    break;
                case 0x7fff: /* crc error */
                    break;
                default:
                    grad = itemp >> 4;
                    migrad = (itemp & 0x0f) * 625;
                    if(temp < 0) {
                        tsign = '-';
                        if(migrad > 0) {
                            migrad = 10000 - migrad;
                            grad = -grad - 1;
                        } else {
                            grad = -grad;
                        }
                    }
                    sprintf(temp, "%c%d.%04d C", tsign, grad, migrad);
                }
            }
        }
    }
    return true;
}

void UNCDevice::ToggleLine(int n)
{
    int action;
    if(n < 4) {
        action = 0x03 << (2*n);
    } else if(n >= 8) {
        action = 0x0200 | (0x03 << (2*(n-8)));
    } else {
        return ; // no operation
    }
    if(!dev_hdl) return;
    unc001_perform_action(dev_hdl, action);
}
